/* Load file, variables */
#include <stdio.h>
#include <string.h>
#include "Illust.h"

#define BUFLEN 1024

/* Variables */
int hsize;
int vsize;
Cell **image;
Cell **hsum;
Cell **vsum;

/*
  Load(filename): Load pattern from file
    File format:
      # comment
      hsize
      vsize
      hsum ...
        :
      vsum ...
        :
*/
void Load(char *filename)
{
  FILE *fp;
  char buf[BUFLEN];
  char *p, *sums, *sum;
  int lines = 0;
  int stat = 0;
  int num;
  int i, j;

  if ((fp = fopen(filename, "r")) == NULL) {
    fprintf(stderr, "Cannot open file '%s'.\n", filename);
    exit(1);
  }
  while (fgets(buf, BUFLEN, fp) != NULL) {
    lines++;
    /*    fprintf(stderr, "%d:%d:%s", stat, lines, buf); */
    if (*buf != '#') {
      switch (stat) {
      case 0: /* hsize */
	if (sscanf(buf, "%d", &hsize) != 1) {
	  fprintf(stderr, "Bad hsize '%s' at %d\n", buf, lines);
	  exit(1);
	}
	stat++;
	break;
      case 1: /* vsize */
	if (sscanf(buf, "%d", &vsize) != 1) {
	  fprintf(stderr, "Bad vsize '%s' at %d\n", buf, lines);
	  exit(1);
	}

	/* Allocate Memory */
	/* Image */
	if ((image = (Cell **)calloc(vsize, sizeof(Cell *))) == NULL) {
	  fprintf(stderr, "Cannot allocate memory for image[%d]\n", vsize);
	  exit(1);
	}
	for (i = 0; i < vsize; i++) {
	  if ((image[i] = (Cell *)calloc(hsize, sizeof(Cell)))
	      == NULL) {
	    fprintf(stderr, "Cannot allocate memory for image[%d][%d])\n",
		    vsize, i);
	    exit(1);
	  }
	  for (j = 0; j < hsize; j++)
	    image[i][j] = CellUnknown;
	}

	/* Horizontal Sum */
	if ((hsum = (Cell **)calloc(vsize, sizeof(Cell *))) == NULL) {
	  fprintf(stderr, "Cannot allocate memory for hsum[%d]\n", vsize);
	  exit(1);
	}
	for (i = 0; i < vsize; i++) {
	  if ((hsum[i] = (Cell *)calloc((hsize + 3) * 2, sizeof(Cell)))
	      == NULL) {
	    fprintf(stderr, "Cannot allocate memory for hsum[%d][%d])\n",
		    vsize, i);
	    exit(1);
	  }
	  for (j = 0; j < (hsize + 3) / 2; j++)
	    hsum[i][j] = CellUnknown;
	}

	/* Vertical Sum */
	if ((vsum = (Cell **)calloc(hsize, sizeof(Cell *))) == NULL) {
	  fprintf(stderr, "Cannot allocate memory vsum[%d]\n", hsize);
	  exit(1);
	}
	for (i = 0; i < hsize; i++) {
	  if ((vsum[i] = (Cell *)calloc((vsize + 3) / 2, sizeof(Cell)))
	      == NULL) {
	    fprintf(stderr, "Cannot allocate memory for vsum[%d][%d]\n",
		    hsize, i);
	    exit(1);
	  }
	  for (j = 0; j < (vsize + 3) / 2; j++)
	    vsum[i][j] = CellUnknown;
	}
	stat++;
	num = 0;
	break;
      case 2: /* Horizontal Sum */
	if (strlen(buf) > 1) {
	  for (p = sums = strdup(buf), i = 0;
	       ((sum = strsep(&sums, ",/:; \t")) != NULL) &&
		 (i < (hsize + 1) / 2);
	       i++) {
	    if (sscanf(sum, "%d", &(hsum[num][i])) != 1) {
	      fprintf(stderr, "Bad sum '%s' for hsum[%d] at %d\n",
		      buf, num, lines);
	      exit(1);
	    }
	  }
	  if (i > (hsize + 1) / 2) {
	    fprintf(stderr, "Too many sums '%s' for hsum[%d] at %d\n",
		    buf, num, lines);
	    exit(1);
	  }
	}
	if (++num == vsize) {
	  stat++;
	  num = 0;
	}
	break;
      case 3: /* Vertical Sum */
	if (strlen(buf) > 1) {
	  for (p = sums = strdup(buf), i = 0;
	       ((sum = strsep(&sums, ",/:; \t")) != NULL) &&
		 (i < (vsize + 1) / 2);
	       i++) {
	    if (sscanf(sum, "%d", &(vsum[num][i])) != 1) {
	      fprintf(stderr, "Bad sum '%s' for vsum[%d] at %d\n",
		      buf, num, lines);
	      exit(1);
	    }
	  }
	  if (i > (vsize + 1) / 2) {
	    fprintf(stderr, "Too many sums '%s' for vsum[%d] at %d\n",
		    buf, num, lines);
	    exit(1);
	  }
	}
	if (++num == hsize) {
	  stat++;
	  num = 0;
	}
	break;
      case 4: /* Too many lines */
	fprintf(stderr, "Too many lines at %d\nIgnored\n", lines);
	stat++;
	break;
      default:
	break;
      }
    }
  }
  fclose(fp);

  if (stat < 4) {
    fprintf(stderr, "Unexpected end of file at %d, step %d\n", lines, stat);
    exit(1);
  }
}
